
#region Header

/************************************************************************************************
Plugin name: FireBall(a little game) 

Write by cjyhot

Version 1.0 2024-03-14

************************************************************************************************/

#endregion


using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace CjyhotHelper
{
    public class FireBall : MVRScript
    {
        private SuperController sc = SuperController.singleton;
        private List<GameObject> sphereList = new List<GameObject>();
        Rigidbody rb;

        private JSONStorableFloat radiusValue;
        private JSONStorableFloat massValue;

        private JSONStorableFloat speedValue;
        private JSONStorableFloat frequencyValue;
        private JSONStorableFloat showSpeedValue;

        //mode 1:set speed mode
        //mode 2:press trigger time to set speed mode
        //mode 3:left trigger value set speed ,right trigger fire ball
        //mode 4:continus fire ball
        private int gameMode = 1;

        private bool startPress = false;
        private float counterSpeed = 0f;
        private float leftValue = 0f;
        private int maxDelayCount = 60;
        private int delayCount = 0;

        private JSONStorableFloat receiverTarget;

        void Start()
        {

            RegisterAction(new JSONStorableAction("ClearAllBalls", ClearAllBalls));

            RegisterAction(new JSONStorableAction("Mode 1", SetMode1));
            RegisterAction(new JSONStorableAction("Mode 2", SetMode2));
            RegisterAction(new JSONStorableAction("Mode 3", SetMode3));
            RegisterAction(new JSONStorableAction("Mode 4", SetMode4));

            speedValue = new JSONStorableFloat("speedValue", 10f, 0f, 30f, true, true);
            RegisterFloat(speedValue);

            radiusValue = new JSONStorableFloat("radiusValue", 0.07f, 0f, 2f, true, true);
            RegisterFloat(radiusValue);

            massValue = new JSONStorableFloat("massValue", 0.5f, 0f, 10f, true, true);
            RegisterFloat(massValue);

            frequencyValue = new JSONStorableFloat("frequencyValue", 20f, 0f, 60f, true, true);
            RegisterFloat(frequencyValue);

            showSpeedValue = new JSONStorableFloat("showSpeedValue", 1f, 0f, 30f, true, true);
            RegisterFloat(showSpeedValue);
            Atom showSpeedSlideAtom = sc.GetAtomByUid("showSpeed");
            var receiver = showSpeedSlideAtom.GetStorableByID("Trigger");
            receiverTarget = receiver.GetFloatJSONParam("value");

        }



        void Update()
        {
            VROperation();
        }

        private void VROperation()
        {
            switch (gameMode)
            {
                case 1:
                    if (sc.GetRightGrab())
                    {
                        Fire(speedValue.val);
                    }
                    break;
                case 2:
                    if (sc.GetRightGrabVal() >= 0.9f)
                    {
                        startPress = true;
                        counterSpeed += 0.02f;
                        showSpeedValue.val = counterSpeed;
                        if (receiverTarget != null)
                        {
                            receiverTarget.val = showSpeedValue.val;
                        }
                        if (counterSpeed >= 1f) counterSpeed = 0f;
                    }
                    else
                    {
                        if (startPress)
                        {
                            startPress = false;
                            Fire(counterSpeed * 30f);
                            counterSpeed = 0f;
                        }
                    }
                    break;
                case 3:
                    showSpeedValue.val = sc.GetLeftGrabVal();
                    if (receiverTarget != null)
                    {
                        receiverTarget.val = showSpeedValue.val;
                    }
                    if (sc.GetRightGrab())
                    {
                        Fire(showSpeedValue.val * 30);
                    }
                    break;
                case 4:
                    if (sc.GetRightGrabVal() >= 0.9f)
                    {
                        maxDelayCount = (int)(60f / frequencyValue.val);
                        delayCount++;
                        if (delayCount >= maxDelayCount)
                        {
                            Fire(speedValue.val);
                            delayCount = 0;
                        }
                    }
                    else
                    {
                        delayCount = 0;
                    }
                    break;
                default:
                    break;
            }

            if (sphereList.Count >= 100)
            {
                GameObject.Destroy(sphereList[0]);
                sphereList.RemoveAt(0);
            }
        }

        private GameObject CreateSphere()
        {
            GameObject sphereGameObject = GameObject.CreatePrimitive(PrimitiveType.Sphere);
            sphereGameObject.transform.localScale = new Vector3(radiusValue.val, radiusValue.val, radiusValue.val);
            SphereCollider sphereCollider = sphereGameObject.AddComponent<SphereCollider>();
            sphereCollider.radius = radiusValue.val;
            sphereCollider.isTrigger = true;
            sphereCollider.material = new PhysicMaterial();
            sphereCollider.material.bounciness = 0.9f;
            sphereCollider.material.staticFriction = 0.5f;
            sphereCollider.material.dynamicFriction = 0.5f;
            Rigidbody sphereRigidBody = sphereGameObject.AddComponent<Rigidbody>();
            sphereRigidBody.mass = massValue.val;
            sphereRigidBody.useGravity = false;
            return sphereGameObject;
        }

        private void Fire(float speed)
        {
            GameObject sphere = CreateSphere();
            sphereList.Add(sphere);
            sphere.transform.position = sc.rightHand.position + sc.rightHand.forward * 0.1f;
            rb = sphere.GetComponent<Rigidbody>();
            rb.useGravity = true;
            rb.velocity = sc.rightHand.forward * speed;
        }

        void ClearAllBalls()
        {
            for (int i = sphereList.Count - 1; i >= 0; i--)
            {
                GameObject.Destroy(sphereList[i]);
                sphereList.RemoveAt(i);
            }
        }

        void SetMode1()
        {
            gameMode = 1;
        }

        void SetMode2()
        {
            gameMode = 2;
        }

        void SetMode3()
        {
            gameMode = 3;
        }

        void SetMode4()
        {
            gameMode = 4;
        }
    }
}
